﻿using AAD_KeyVault_Certificate_Authentication.Abstractions;
using AAD_KeyVault_Certificate_Authentications.EntityEnumerators;
using Microsoft.Graph;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;

namespace AAD_KeyVault_Certificate_Authentications
{
    public class ClientGraph : IGraphClient
    {
        private readonly GraphServiceClient graphClient;

        public ClientGraph(IGraphClientFactory graphClientFactory)
        {
            this.graphClient = graphClientFactory.CreateClient();
        }

        public async Task<Group> GetGroupsByIdAsync(Guid groupId)
        {
            try
            {
                IGraphServiceGroupsCollectionPage groupsPage = await
                     graphClient.Groups
                     .Request()
                     .Filter($"id eq '{groupId}'")
                     .Select("id, displayName")
                     .GetAsync();

                if (groupsPage.CurrentPage.Count < 1)
                {
                    return null;
                }

                return groupsPage.CurrentPage[0];
            }
            catch (Exception ex)
            {
                bool handled = false;
                if (ex is ServiceException svcEx)
                {
                    if (svcEx.StatusCode == System.Net.HttpStatusCode.NotFound)
                    {
                        handled = true;
                    }
                }

                if (!handled)
                {
                    throw;
                }
                else
                {
                    return null;
                }
            }
        }

        public async Task<User> GetUserByIdAsync(Guid userId)
        {
            IGraphServiceUsersCollectionPage usersPage = await graphClient.Users
               .Request()
               .Filter($"id eq '{userId}'")
               .Select("id, displayName,userPrincipalName, memberOf")
               .Expand("memberOf")
               .GetAsync();

            if (usersPage.CurrentPage.Count < 1)
            {
                return null;
            }

            return usersPage.CurrentPage[0];
        }

        /// <summary>
        /// https://docs.microsoft.com/en-us/graph/api/user-list?view=graph-rest-1.0&tabs=http
        /// <para>GET /users?$select=id, displayName,mail,userPrincipalName,onPremisesExtensionAttributesx</para>
        /// <para>Note: Skip does not work. Therefore we need to iterate through all user pages</para>
        /// </summary>
        public IAsyncEnumerable<User> EnumerateUsersAsync()
        {
            // the graph will return all users in batches and we have to manually move to next page
            // the BatchAsyncEnumerable will automatically do that for us
            BatchUsersGraphProvider batchGraphProvider = new BatchUsersGraphProvider(graphClient);
            return new BatchAsyncEnumerable<User>(batchGraphProvider);
        }

        public IAsyncEnumerable<Group> EnumerateGroupsAsync()
        {
            // the graph will return all users in batches and we have to manually move to next page
            // the BatchAsyncEnumerable will automatically do that for us
            BatchGroupsGraphProvider batchGraphProvider = new BatchGroupsGraphProvider(graphClient);
            return new BatchAsyncEnumerable<Group>(batchGraphProvider);
        }

        /// <summary>
        /// DELETE /groups/{id}/members/{id}/$ref
        /// https://docs.microsoft.com/en-us/graph/api/group-delete-members?view=graph-rest-1.0&tabs=csharp#http-request
        /// </summary>
        public async Task RemoveGroupMember(Guid groupId, Guid memberId)
        {
            await graphClient.Groups[groupId.ToString()].Members[memberId.ToString()].Reference
                  .Request()
                  .DeleteAsync();
        }

    }
}
