﻿using Azure;
using Azure.Core;
using Azure.Identity;
using Azure.Security.KeyVault.Certificates;
using Azure.Security.KeyVault.Secrets;
using AAD_KeyVault_Certificate_Authentication.Abstractions;
using Microsoft.Graph;
using Microsoft.Graph.Auth;
using Microsoft.Identity.Client;
using System;
using System.Security.Cryptography.X509Certificates;

namespace AAD_KeyVault_Certificate_Authentications
{
    public class ConfidentialClientFactory : IGraphClientFactory
    {
        private readonly IConfigurationGraph configuration;

        public ConfidentialClientFactory(IConfigurationGraph configuration)
        {
            this.configuration = configuration;
        }

        public GraphServiceClient CreateClient()
        {
            // using Azure package instead of Microsoft.Azure
            // https://github.com/Azure/azure-sdk-for-net/blob/d9d93df6c75797a6027c125ab3ff61b2ac894102/sdk/keyvault/Azure.Security.KeyVault.Secrets/CHANGELOG.md#major-changes-from-microsoftazurekeyvault
            // https://stackoverflow.com/questions/59849541/azure-security-keyvault-secrets-vs-microsoft-azure-keyvault

            TokenCredential credentials = configuration.UseAzureCliCredentials
                ? new AzureCliCredential() // for debugging we login uzing azure cli to be able to access the KeyVault
                : new DefaultAzureCredential();

            CertificateClient client = new CertificateClient(
                vaultUri: new Uri(configuration.KeyVaultUri),
                credential: credentials);

            SecretClient secretClient = new SecretClient(
                vaultUri: new Uri(configuration.KeyVaultUri),
                credential: credentials);

            Pageable<CertificateProperties> versions
                = client.GetPropertiesOfCertificateVersions(configuration.CertificateName);

            X509Certificate2 x509Certificate2 = null;

            // https://stackoverflow.com/questions/37033073/how-can-i-create-an-x509certificate2-object-from-an-azure-key-vault-keybundle
            foreach (CertificateProperties certificate in versions)
            {
                if (!(certificate.Enabled ?? false) ||
                    certificate.ExpiresOn <= DateTimeOffset.UtcNow)
                {
                    continue;
                }

                KeyVaultSecret certificateSecret = secretClient.GetSecret(certificate.Name, certificate.Version).Value;
                byte[] privateKey = Convert.FromBase64String(certificateSecret.Value);
                x509Certificate2 = new X509Certificate2(privateKey, (string)null, X509KeyStorageFlags.MachineKeySet);
                break;
            }

            if (x509Certificate2 is null)
            {
                throw new Exception("No valid certificate found");
            }

            //https://docs.microsoft.com/en-us/graph/sdks/choose-authentication-providers?tabs=CS#ClientCredentialsProvider
            IConfidentialClientApplication confidentialClientApplication = ConfidentialClientApplicationBuilder
                .Create(configuration.ClientId.ToString())
                .WithTenantId(configuration.TenantId.ToString())
                .WithCertificate(x509Certificate2)
                .Build();

            ClientCredentialProvider authProvider = new ClientCredentialProvider(confidentialClientApplication);

            // Create a new instance of GraphServiceClient with the authentication provider.
            return new GraphServiceClient(authProvider);
        }
    }
}
